@@ -66,7 +66,7 @@ const pthread_t &Thread::get_thread_id()
   return thread_id;
 }
 
-bool Thread::is_started()
+bool Thread::is_started() const
 {
   return thread_id != 0;
 }
@@ -87,11 +87,11 @@ int Thread::kill(int signal)
 int Thread::try_create(size_t stacksize)
 {
   pthread_attr_t *thread_attr = NULL;
-  pthread_attr_t thread_attr_loc;
-  
   stacksize &= CEPH_PAGE_MASK;  // must be multiple of page
   if (stacksize) {
-    thread_attr = &thread_attr_loc;
+    thread_attr = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
+    if (!thread_attr)
+      return -ENOMEM;
     pthread_attr_init(thread_attr);
     pthread_attr_setstacksize(thread_attr, stacksize);
   }
@@ -113,10 +113,8 @@ int Thread::try_create(size_t stacksize)
   r = pthread_create(&thread_id, thread_attr, _entry_func, (void*)this);
   restore_sigset(&old_sigset);
 
-  if (thread_attr) {
-    pthread_attr_destroy(thread_attr);	
-  }
-
+  if (thread_attr)
+    free(thread_attr);
   return r;
 }
 
