@@ -71,6 +71,10 @@ void RecoveryQueue::advance()
       break;
     }
   }
+
+  logger->set(l_mdc_num_recovering_processing, file_recovering.size());
+  logger->set(l_mdc_num_recovering_enqueued, file_recover_queue.size());
+  logger->set(l_mdc_num_recovering_prioritized, file_recover_queue_front.size());
 }
 
 void RecoveryQueue::_start(CInode *in)
@@ -110,6 +114,7 @@ void RecoveryQueue::prioritize(CInode *in)
   if (file_recover_queue.count(in)) {
     dout(20) << *in << dendl;
     file_recover_queue_front.insert(in);
+    logger->set(l_mdc_num_recovering_prioritized, file_recover_queue_front.size());
     return;
   }
 
@@ -124,14 +129,17 @@ void RecoveryQueue::prioritize(CInode *in)
 void RecoveryQueue::enqueue(CInode *in)
 {
   dout(15) << "RecoveryQueue::enqueue " << *in << dendl;
+  assert(logger);  // Caller should have done set_logger before using me
   assert(in->is_auth());
 
   in->state_clear(CInode::STATE_NEEDSRECOVER);
   if (!in->state_test(CInode::STATE_RECOVERING)) {
     in->state_set(CInode::STATE_RECOVERING);
     in->auth_pin(this);
+    logger->inc(l_mdc_recovery_started);
   }
   file_recover_queue.insert(in);
+  logger->set(l_mdc_num_recovering_enqueued, file_recover_queue.size());
 }
 
 
@@ -153,6 +161,8 @@ void RecoveryQueue::_recovered(CInode *in, int r, uint64_t size, utime_t mtime)
   }
 
   file_recovering.erase(in);
+  logger->set(l_mdc_num_recovering_processing, file_recovering.size());
+  logger->inc(l_mdc_recovery_completed);
   in->state_clear(CInode::STATE_RECOVERING);
 
   if (!in->get_parent_dn() && !in->get_projected_parent_dn()) {
