@@ -22,7 +22,7 @@
 
 class MMDSResolveAck : public Message {
  public:
-  vector<metareqid_t> commit;
+  map<metareqid_t, bufferlist> commit;
   vector<metareqid_t> abort;
 
   MMDSResolveAck() : Message(MSG_MDS_RESOLVEACK) {}
@@ -30,8 +30,8 @@ private:
   ~MMDSResolveAck() {}
 
 public:
-  const char *get_type_name() { return "resolve_ack"; }
-  /*void print(ostream& out) {
+  const char *get_type_name() const { return "resolve_ack"; }
+  /*void print(ostream& out) const {
     out << "resolve_ack.size()
 	<< "+" << ambiguous_imap.size()
 	<< " imports +" << slave_requests.size() << " slave requests)";
@@ -39,17 +39,17 @@ public:
   */
   
   void add_commit(metareqid_t r) {
-    commit.push_back(r);
+    commit[r].clear();
   }
   void add_abort(metareqid_t r) {
     abort.push_back(r);
   }
 
-  void encode_payload(CephContext *cct) {
+  void encode_payload(uint64_t features) {
     ::encode(commit, payload);
     ::encode(abort, payload);
   }
-  void decode_payload(CephContext *cct) {
+  void decode_payload() {
     bufferlist::iterator p = payload.begin();
     ::decode(commit, p);
     ::decode(abort, p);
