@@ -379,8 +379,8 @@ private:
     full_state(FULL_NOTFULL),
     fd(-1),
     writing_seq(0),
-    throttle_ops(g_ceph_context, "filestore_ops"),
-    throttle_bytes(g_ceph_context, "filestore_bytes"),
+    throttle_ops(g_ceph_context, "filestore_ops", g_conf->journal_queue_max_ops),
+    throttle_bytes(g_ceph_context, "filestore_bytes", g_conf->journal_queue_max_bytes),
     write_lock("FileJournal::write_lock", false, true, false, g_ceph_context),
     write_stop(false),
     aio_stop(false),
