@@ -30,6 +30,38 @@ using namespace std;
 struct req_state;
 class RGWHandler;
 
+enum RGWOpType {
+  RGW_OP_UNKNOWN = 0,
+  RGW_OP_GET_OBJ,
+  RGW_OP_LIST_BUCKETS,
+  RGW_OP_STAT_ACCOUNT,
+  RGW_OP_LIST_BUCKET,
+  RGW_OP_GET_BUCKET_LOGGING,
+  RGW_OP_GET_BUCKET_VERSIONING,
+  RGW_OP_SET_BUCKET_VERSIONING,
+  RGW_OP_STAT_BUCKET,
+  RGW_OP_CREATE_BUCKET,
+  RGW_OP_DELETE_BUCKET,
+  RGW_OP_PUT_OBJ,
+  RGW_OP_POST_OBJ,
+  RGW_OP_PUT_METADATA,
+  RGW_OP_SET_TEMPURL,
+  RGW_OP_DELETE_OBJ,
+  RGW_OP_COPY_OBJ,
+  RGW_OP_GET_ACLS,
+  RGW_OP_PUT_ACLS,
+  RGW_OP_GET_CORS,
+  RGW_OP_PUT_CORS,
+  RGW_OP_DELETE_CORS,
+  RGW_OP_OPTIONS_CORS,
+  RGW_OP_INIT_MULTIPART,
+  RGW_OP_COMPLETE_MULTIPART,
+  RGW_OP_ABORT_MULTIPART,
+  RGW_OP_LIST_MULTIPART,
+  RGW_OP_LIST_BUCKET_MULTIPARTS,
+  RGW_OP_DELETE_MULTI_OBJ,
+};
+
 /**
  * Provide the base class for all ops.
  */
@@ -75,6 +107,7 @@ public:
     send_response();
   }
   virtual const string name() = 0;
+  virtual RGWOpType get_type() { return RGW_OP_UNKNOWN; }
 
   virtual uint32_t op_mask() { return 0; }
 };
@@ -141,6 +174,7 @@ public:
   virtual int send_response_data(bufferlist& bl, off_t ofs, off_t len) = 0;
 
   virtual const string name() { return "get_obj"; }
+  virtual RGWOpType get_type() { return RGW_OP_GET_OBJ; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -171,6 +205,7 @@ public:
   virtual bool should_get_stats() { return false; }
 
   virtual const string name() { return "list_buckets"; }
+  virtual RGWOpType get_type() { return RGW_OP_LIST_BUCKETS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -196,16 +231,18 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "stat_account"; }
+  virtual RGWOpType get_type() { return RGW_OP_STAT_ACCOUNT; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWListBucket : public RGWOp {
 protected:
   string prefix;
-  string marker; 
-  string next_marker; 
+  rgw_obj_key marker; 
+  rgw_obj_key next_marker; 
   string max_keys;
   string delimiter;
+  bool list_versions;
   int max;
   int ret;
   vector<RGWObjEnt> objs;
@@ -217,12 +254,8 @@ protected:
   int parse_max_keys();
 
 public:
-  RGWListBucket() {
-    max = 0;
-    ret = 0;
-    default_max = 0;
-    is_truncated = false;
-  }
+  RGWListBucket() : list_versions(false), max(0), ret(0),
+                    default_max(0), is_truncated(false) {}
   int verify_permission();
   void pre_exec();
   void execute();
@@ -230,6 +263,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "list_bucket"; }
+  virtual RGWOpType get_type() { return RGW_OP_LIST_BUCKET; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -241,6 +275,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "get_bucket_logging"; }
+  virtual RGWOpType get_type() { return RGW_OP_GET_BUCKET_LOGGING; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -256,6 +291,41 @@ public:
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
+class RGWGetBucketVersioning : public RGWOp {
+protected:
+  bool versioned;
+  bool versioning_enabled;
+public:
+  RGWGetBucketVersioning() : versioned(false), versioning_enabled(false) {}
+
+  int verify_permission();
+  void pre_exec();
+  void execute();
+
+  virtual void send_response() = 0;
+  virtual const string name() { return "get_bucket_versioning"; }
+  virtual RGWOpType get_type() { return RGW_OP_GET_BUCKET_VERSIONING; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
+};
+
+class RGWSetBucketVersioning : public RGWOp {
+protected:
+  bool enable_versioning;
+  int ret;
+public:
+  RGWSetBucketVersioning() : enable_versioning(false), ret(0) {}
+
+  int verify_permission();
+  void pre_exec();
+  void execute();
+
+  virtual int get_params() { return 0; }
+
+  virtual void send_response() = 0;
+  virtual const string name() { return "set_bucket_versioning"; }
+  virtual RGWOpType get_type() { return RGW_OP_SET_BUCKET_VERSIONING; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
+};
 
 class RGWStatBucket : public RGWOp {
 protected:
@@ -272,6 +342,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "stat_bucket"; }
+  virtual RGWOpType get_type() { return RGW_OP_STAT_BUCKET; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -301,6 +372,7 @@ public:
   virtual int get_params() { return 0; }
   virtual void send_response() = 0;
   virtual const string name() { return "create_bucket"; }
+  virtual RGWOpType get_type() { return RGW_OP_CREATE_BUCKET; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -319,6 +391,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "delete_bucket"; }
+  virtual RGWOpType get_type() { return RGW_OP_DELETE_BUCKET; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
@@ -331,6 +404,8 @@ protected:
   off_t ofs;
   const char *supplied_md5_b64;
   const char *supplied_etag;
+  const char *if_match;
+  const char *if_nomatch;
   string etag;
   bool chunked_upload;
   RGWAccessControlPolicy policy;
@@ -339,16 +414,22 @@ protected:
 
   MD5 *user_manifest_parts_hash;
 
+  uint64_t olh_epoch;
+  string version_id;
+
 public:
   RGWPutObj() {
     ret = 0;
     ofs = 0;
     supplied_md5_b64 = NULL;
     supplied_etag = NULL;
+    if_match = NULL;
+    if_nomatch = NULL;
     chunked_upload = false;
     obj_manifest = NULL;
     mtime = 0;
     user_manifest_parts_hash = NULL;
+    olh_epoch = 0;
   }
 
   virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
@@ -356,7 +437,7 @@ public:
     policy.set_ctx(s->cct);
   }
 
-  RGWPutObjProcessor *select_processor(bool *is_multipart);
+  RGWPutObjProcessor *select_processor(RGWObjectCtx& obj_ctx, bool *is_multipart);
   void dispose_processor(RGWPutObjProcessor *processor);
 
   int verify_permission();
@@ -367,6 +448,7 @@ public:
   virtual int get_data(bufferlist& bl) = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "put_obj"; }
+  virtual RGWOpType get_type() { return RGW_OP_PUT_OBJ; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -403,13 +485,14 @@ public:
   void pre_exec();
   void execute();
 
-  RGWPutObjProcessor *select_processor();
+  RGWPutObjProcessor *select_processor(RGWObjectCtx& obj_ctx);
   void dispose_processor(RGWPutObjProcessor *processor);
 
   virtual int get_params() = 0;
   virtual int get_data(bufferlist& bl) = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "post_obj"; }
+  virtual RGWOpType get_type() { return RGW_OP_POST_OBJ; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -420,6 +503,7 @@ protected:
   bool has_policy, has_cors;
   RGWAccessControlPolicy policy;
   RGWCORSConfiguration cors_config;
+  string placement_rule;
 
 public:
   RGWPutMetadata() {
@@ -439,6 +523,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "put_obj_metadata"; }
+  virtual RGWOpType get_type() { return RGW_OP_PUT_METADATA; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -455,14 +540,17 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "set_temp_url"; }
+  virtual RGWOpType get_type() { return RGW_OP_SET_TEMPURL; }
 };
 
 class RGWDeleteObj : public RGWOp {
 protected:
   int ret;
+  bool delete_marker;
+  string version_id;
 
 public:
-  RGWDeleteObj() : ret(0) {}
+  RGWDeleteObj() : ret(0), delete_marker(false) {}
 
   int verify_permission();
   void pre_exec();
@@ -470,6 +558,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "delete_obj"; }
+  virtual RGWOpType get_type() { return RGW_OP_DELETE_OBJ; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
@@ -491,7 +580,7 @@ protected:
   map<string, bufferlist> attrs;
   string src_bucket_name;
   rgw_bucket src_bucket;
-  string src_object;
+  rgw_obj_key src_object;
   string dest_bucket_name;
   rgw_bucket dest_bucket;
   string dest_object;
@@ -506,6 +595,9 @@ protected:
 
   off_t last_ofs;
 
+  string version_id;
+  uint64_t olh_epoch;
+
 
   int init_common();
 
@@ -526,9 +618,10 @@ public:
     mtime = 0;
     replace_attrs = false;
     last_ofs = 0;
+    olh_epoch = 0;
   }
 
-  static bool parse_copy_location(const char *src, string& bucket_name, string& object);
+  static bool parse_copy_location(const string& src, string& bucket_name, rgw_obj_key& object);
 
   virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
     RGWOp::init(store, s, h);
@@ -544,6 +637,7 @@ public:
   virtual void send_partial_response(off_t ofs) {}
   virtual void send_response() = 0;
   virtual const string name() { return "copy_obj"; }
+  virtual RGWOpType get_type() { return RGW_OP_COPY_OBJ; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -561,6 +655,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "get_acls"; }
+  virtual RGWOpType get_type() { return RGW_OP_GET_ACLS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -569,6 +664,7 @@ protected:
   int ret;
   size_t len;
   char *data;
+  ACLOwner owner;
 
 public:
   RGWPutACLs() {
@@ -588,6 +684,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "put_acls"; }
+  virtual RGWOpType get_type() { return RGW_OP_PUT_ACLS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -603,6 +700,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "get_cors"; }
+  virtual RGWOpType get_type() { return RGW_OP_GET_CORS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -623,6 +721,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "put_cors"; }
+  virtual RGWOpType get_type() { return RGW_OP_PUT_CORS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -638,6 +737,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "delete_cors"; }
+  virtual RGWOpType get_type() { return RGW_OP_DELETE_CORS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -658,6 +758,7 @@ public:
   void get_response_params(string& allowed_hdrs, string& exp_hdrs, unsigned *max_age);
   virtual void send_response() = 0;
   virtual const string name() { return "options_cors"; }
+  virtual RGWOpType get_type() { return RGW_OP_OPTIONS_CORS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -683,6 +784,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "init_multipart"; }
+  virtual RGWOpType get_type() { return RGW_OP_INIT_MULTIPART; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -711,6 +813,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "complete_multipart"; }
+  virtual RGWOpType get_type() { return RGW_OP_COMPLETE_MULTIPART; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
@@ -727,6 +830,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const string name() { return "abort_multipart"; }
+  virtual RGWOpType get_type() { return RGW_OP_ABORT_MULTIPART; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
@@ -759,6 +863,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "list_multipart"; }
+  virtual RGWOpType get_type() { return RGW_OP_LIST_MULTIPART; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -866,6 +971,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const string name() { return "list_bucket_multiparts"; }
+  virtual RGWOpType get_type() { return RGW_OP_LIST_BUCKET_MULTIPARTS; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
@@ -880,7 +986,6 @@ protected:
   bool quiet;
   bool status_dumped;
 
-
 public:
   RGWDeleteMultiObj() {
     ret = 0;
@@ -897,9 +1002,11 @@ public:
   virtual int get_params() = 0;
   virtual void send_status() = 0;
   virtual void begin_response() = 0;
-  virtual void send_partial_response(pair<string,int>& result) = 0;
+  virtual void send_partial_response(rgw_obj_key& key, bool delete_marker,
+                                     const string& marker_version_id, int ret) = 0;
   virtual void end_response() = 0;
   virtual const string name() { return "multi_object_delete"; }
+  virtual RGWOpType get_type() { return RGW_OP_DELETE_MULTI_OBJ; }
   virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
