@@ -29,6 +29,7 @@ public:
     }
 };
 
+#pragma GCC diagnostic ignored "-Wpragmas"
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 
@@ -67,6 +68,7 @@ test_loop(Rados &cluster, std::string pool_name, std::string obj_name)
 }
 
 #pragma GCC diagnostic pop
+#pragma GCC diagnostic warning "-Wpragmas"
 
 void
 test_replicated(Rados &cluster, std::string pool_name, std::string obj_name)
