@@ -85,7 +85,7 @@ int Dumper::dump(const char *dump_file)
 
   cout << "journal is " << start << "~" << len << std::endl;
 
-  Filer filer(objecter);
+  Filer filer(objecter, &finisher);
   bufferlist bl;
 
   C_SaferCond cond;
@@ -113,6 +113,7 @@ int Dumper::dump(const char *dump_file)
     r = safe_write(fd, buf, sizeof(buf));
     if (r) {
       derr << "Error " << r << " (" << cpp_strerror(r) << ") writing journal file header" << dendl;
+      ::close(fd);
       return r;
     }
 
@@ -189,12 +190,14 @@ int Dumper::undump(const char *dump_file)
   if (trimmed_pos > start) {
     derr << std::hex << "Invalid header (trimmed 0x" << trimmed_pos
       << " > expire 0x" << start << std::dec << dendl;
+    ::close(fd);
     return -EINVAL;
   }
 
   if (start > write_pos) {
     derr << std::hex << "Invalid header (expire 0x" << start
       << " > write 0x" << write_pos << std::dec << dendl;
+    ::close(fd);
     return -EINVAL;
   }
 
@@ -231,10 +234,11 @@ int Dumper::undump(const char *dump_file)
   r = header_cond.wait();
   if (r != 0) {
     derr << "Failed to write header: " << cpp_strerror(r) << dendl;
+    ::close(fd);
     return r;
   }
 
-  Filer filer(objecter);
+  Filer filer(objecter, &finisher);
 
   /* Erase any objects at the end of the region to which we shall write
    * the new log data.  This is to avoid leaving trailing junk after
@@ -274,6 +278,7 @@ int Dumper::undump(const char *dump_file)
     r = write_cond.wait();
     if (r != 0) {
       derr << "Failed to write header: " << cpp_strerror(r) << dendl;
+      ::close(fd);
       return r;
     }
       
